# :scroll: List of Graph Filters
*This file is automatically generated with `docgenerator.py`.*

The following filters can be imported from the package `pygrank.algorithms`.
Constructor details are provided, including arguments inherited from and passed to parent classes.
All of them can be used through the code patterns presented at the library's [documentation](documentation.md#graph-filters). 
1. [GenericGraphFilter](#closedformgraphfilter-genericgraphfilter)
2. [HeatKernel](#closedformgraphfilter-heatkernel)
3. [AbsorbingWalks](#recursivegraphfilter-absorbingwalks)
4. [PageRank](#recursivegraphfilter-pagerank)

### <kbd>ClosedFormGraphFilter</kbd> GenericGraphFilter

Implements a graph filter with a specific vector of weight parameters. The constructor initializes the graph filter. 

Args: 
 * *weights:* Optional. A list-like object with elements weights[n] proportional to the importance of propagating personalization graph signals n hops away. If None (default) then [0.9]*10 is used. 

Example:

```python 
from pygrank import GenericGraphFilter 
algorithm = GenericGraphFilter([0.5, 0.25, 0.125], tol=1.E-9) # tol passed to ConvergenceManager 
```

### <kbd>ClosedFormGraphFilter</kbd> HeatKernel

Heat kernel filter. The constructor initializes the HeatKernel filter parameters. 

Args: 
 * *t:* Optional. How many hops until the importance of new nodes starts decreasing. Default value is 5. 

Example:

```python 
from pygrank.algorithms import HeatKernel 
algorithm = HeatKernel(t=3, tol=1.E-9) # tol passed to the ConvergenceManager 
graph, seed_nodes = ... 
ranks = algorithm(graph, {v: 1 for v in seed_nodes}) 
```

### <kbd>RecursiveGraphFilter</kbd> AbsorbingWalks

Implementation of partial absorbing random walks for Lambda = (1-alpha)/alpha diag(absorption vector) . The constructor initializes the AbsorbingWalks filter parameters. For appropriate parameter values. This can model PageRank 
but is in principle a generalization that allows custom absorption rate per node (when not given, these are I). 

Args: 
 * *alpha:* Optional. (1-alpha)/alpha is the absorption rate of the random walk multiplied with individual node absorption rates. This is chosen to yield the same underlying meaning as PageRank (for which Lambda = alpha Diag(degrees) ) when the same parameter value alpha is chosen. Default is 1-1.E-6 per the respective publication. 

Example:

```python 
from pygrank.algorithms import AbsorbingWalks 
algorithm = AbsorbingWalks(1-1.E-6, tol=1.E-9) # tol passed to the ConvergenceManager 
graph, seed_nodes = ... 
ranks = algorithm(graph, {v: 1 for v in seed_nodes}) 
```


Example (same outcome, explicit absorption rate definition):

```python 
from pygrank.algorithms import AbsorbingWalks 
algorithm = AbsorbingWalks(1-1.E-6, tol=1.E-9) # tol passed to the ConvergenceManager 
graph, seed_nodes = ... 
ranks = algorithm(graph, {v: 1 for v in seed_nodes}, absorption={v: 1 for v in graph}) 
```

### <kbd>RecursiveGraphFilter</kbd> PageRank

A Personalized PageRank power method algorithm. The constructor initializes the PageRank scheme parameters. 

Args: 
 * *alpha:* Optional. 1-alpha is the bias towards the personalization. Default value is 0.85. 

Example:

```python 
import pygrank as pg 
algorithm = pg.PageRank(alpha=0.99, tol=1.E-9) # tol passed to the ConvergenceManager 
graph, seed_nodes = ... 
ranks = algorithm(graph, {v: 1 for v in seed_nodes}) 
```
